#NoTrayIcon
#AutoIt3Wrapper_icon=VolumeOSD.ico
#AutoIt3Wrapper_Res_Icon_Add=lock.ico
#AutoIt3Wrapper_Res_Icon_Add=mute.ico
#AutoIt3Wrapper_Res_Icon_Add=mute_lock.ico
#AutoIt3Wrapper_Res_Comment=OSD Volume Control
#AutoIt3Wrapper_Res_Description=VolumeOSD
#AutoIt3Wrapper_Res_Fileversion=1.1.0.0
#AutoIt3Wrapper_Res_LegalCopyright=by Erik Pilsits
#AutoIt3Wrapper_Res_Language=1033
#AutoIt3Wrapper_Res_requestedExecutionLevel=asInvoker
#AutoIt3Wrapper_AU3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
#AutoIt3Wrapper_Run_Obfuscator=y
#Obfuscator_Parameters=/sf /sv /om /cs=0 /cn=0
#AutoIt3Wrapper_Run_After=del "%scriptdir%\%scriptfile%_Obfuscated.au3"

#include <GuiConstantsEx.au3>
#include <ProgressConstants.au3>
#include <WindowsConstants.au3>
#include <Timers.au3>
#include <Misc.au3>
#include <WinAPI.au3>
#include <StaticConstants.au3>
#include <_AudioEndpointVolume.au3>

Opt("GUIOnEventMode", 1)
Opt("TrayOnEventMode", 1)
Opt("MustDeclareVars", 1)
Opt("TrayMenuMode", 3)

Global $Volume_OSD_MSG = _WinAPI_RegisterWindowMessage("Volume_OSD_MSG")

If _Singleton("VolumeOSD_Mutex", 1) = 0 Then ; another instance
	If $CmdLine[0] = 0 Then Exit ; exit if no cmd args
	If $CmdLine[1] = "exit" Then _SendMessage(WinGetHandle("VolumeOSD_GUI"), $Volume_OSD_MSG, 1, 0) ; exit remote prog
	Exit
EndIf

TraySetState()
TraySetClick(16)
Global $tray_lock = TrayCreateItem("Lock Volume")
TrayItemSetOnEvent(-1, "_LockVolume")
TrayCreateItem("")
TrayCreateItem("Exit")
TrayItemSetOnEvent(-1, "_Exit")

HotKeySet("{VOLUME_UP}", "_vol_up")
HotKeySet("{VOLUME_DOWN}", "_vol_down")
HotKeySet("{VOLUME_MUTE}", "_vol_mute")

Global $gui, $prog, $label, $th, $show, $timer = -1
Global $transColor = 0x232323
Global $locked = False

_SetTrayIcon()
_SetTrayToolTip()
$th = WinGetPos("[CLASS:Shell_TrayWnd]")
$th = $th[3]

$gui = GUICreate("VolumeOSD_GUI", 40, 150, @DesktopWidth - 8 - 40, @DesktopHeight - $th - 8 - 150, $WS_POPUP, BitOR($WS_EX_TOOLWINDOW, $WS_EX_LAYERED))
GUISetBkColor($transColor)
$prog = GUICtrlCreateProgress(28, 0, 12, 150, BitOR($PBS_SMOOTH, $PBS_VERTICAL))
$label = GUICtrlCreateLabel("", 0, 135, 25, 20, $SS_RIGHT)
GUICtrlSetFont(-1, 10, 600)
GUICtrlSetColor(-1, 0xDDDDDD)
GUICtrlSetBkColor(-1, 0x000000)
WinSetOnTop($gui, "", 1)
_WinAPI_SetLayeredWindowAttributes($gui, $transColor, 0, 0x03, True)

GUISetOnEvent($GUI_EVENT_CLOSE, "_Exit")
GUIRegisterMsg($WM_TIMER, "_MY_WM_TIMER")
GUIRegisterMsg($Volume_OSD_MSG, "_MY_CUSTOM_MSG")

GUISetState()

While 1
	Sleep(1000)
WEnd

Func _MY_WM_TIMER($hWnd, $iMsg, $iwParam, $ilParam)
	#forceref $iMsg, $ilParam
    Switch $hWnd
		Case $gui
			Switch _Timer_GetTimerID($iwParam)
				Case $timer
					_ProgHide()
			EndSwitch
	EndSwitch
    Return $GUI_RUNDEFMSG
EndFunc

Func _SetTrayToolTip()
	Local $text = "Volume: "
	If _GetMute() Then
		$text &= "Muted"
		If $locked Then $text &= @CRLF & "[Locked]"
	Else
		$text &= Round(_GetMasterVolumeLevelScalar()) & "%"
		If $locked Then $text &= @CRLF & "[Locked]"
	EndIf
	TraySetToolTip($text)
EndFunc

Func _SetTrayIcon()
	; -1 = normal, -5 = lock, -6 = mute, -7 = mute+lock
	Local $icon = -1
	If _GetMute() Then
		If $locked Then
			$icon = -7
		Else
			$icon = -6
		EndIf
	Else
		If $locked Then $icon = -5
	EndIf
	TraySetIcon(@ScriptFullPath, $icon)
EndFunc

Func _LockVolume()
	$locked = Not $locked
	; checked = 1, not checked = 4
	TrayItemSetState($tray_lock, (4 * Number(Not $locked)) + Number($locked))
	_SetTrayIcon()
	_SetTrayToolTip()
EndFunc

Func _vol_up()
	If $locked Then Return
	Local $vol = _GetMasterVolumeLevelScalar() + 1
	If $vol > 100 Then $vol = 100
	_SetVolume($vol)
EndFunc

Func _vol_down()
	If $locked Then Return
	Local $vol = _GetMasterVolumeLevelScalar() - 1
	If $vol < 0 Then $vol = 0
	_SetVolume($vol)
EndFunc

Func _vol_mute()
	If $locked Then Return
	_SetMute(Number(Not _GetMute()))
	_SetTrayIcon()
	_SetTrayToolTip()
	If Not _GetMute() Then _SetVolume(_GetMasterVolumeLevelScalar())
EndFunc

Func _SetVolume($vol)
	If _GetMute() Then
		_SetMute(0)
		TraySetIcon(@ScriptFullPath, -1)
	EndIf
	Local $rvol = Round($vol)
	_SetMasterVolumeLevelScalar($rvol)
	GUICtrlSetData($prog, $rvol)
	GUICtrlSetData($label, $rvol)
	_SetTrayToolTip()
	If Not $show Then _ProgShow()
	$timer = _Timer_SetTimer($gui, 3000, "", $timer)
EndFunc

Func _ProgShow()
	$show = True
	WinSetOnTop($gui, "", 1)
	For $i = 0 To 255 Step 20
		_WinAPI_SetLayeredWindowAttributes($gui, $transColor, $i, 0x03)
		Sleep(10)
	Next
	_WinAPI_SetLayeredWindowAttributes($gui, $transColor, 255, 0x03)
EndFunc

Func _ProgHide()
	For $i = 255 To 0 Step -10
		_WinAPI_SetLayeredWindowAttributes($gui, $transColor, $i, 0x03)
		Sleep(10)
	Next
	_WinAPI_SetLayeredWindowAttributes($gui, $transColor, 0, 0x03)
	_Timer_KillAllTimers($gui)
	$timer = -1
	$show = False
EndFunc

Func _MY_CUSTOM_MSG($hwnd, $msg, $wparam, $lparam)
	#forceref $msg, $lparam
	Switch $hwnd
		Case $gui
			Switch $wparam
				Case 1
					_Exit()
			EndSwitch
	EndSwitch
EndFunc

Func _Exit()
	Exit
EndFunc
